
/**
 * Determines if an array contains a specified value.
 * @param array - the array to search
 * @param value - the value to search for
 * @returns true if the array contains the value; otherwise, false
 * @public
 */
export declare function contains<T>(array: T[], value: T): boolean;

/**
 * Returns the number of values in an array.
 * @param array - the array to count the values of
 * @returns the number of values in the array
 * @public
 */
export declare function count<T>(array: T[]): number;

/**
 * Returns the number of values in an array that satisfy a condition.
 * @param array - the array to count the values of
 * @param callback - the function to test each value for a condition
 * @returns the number of values in the array that satisfy the condition
 * @public
 */
export declare function count<T>(array: T[], callback: (value: T) => boolean): number;

/**
 * Filters the values in an array asynchronously.
 * @param array - the array to filter
 * @param callback - the asynchronous filter function
 * @returns the filtered array
 * @public
 */
export declare function filterAsync<T>(// tslint:disable-line:export-name
array: T[], callback: (value: T, index: number, array: T[]) => Promise<boolean>): Promise<T[]>;

/**
 * Returns the frequency of each value in an array.
 * @param array - the array to get the frequencies of
 * @returns the frequency of each value in an array
 * @public
 */
export declare function frequencies<T>(array: T[]): Map<T, number>;

/**
 * Maps the values in an array asynchronously.
 * @param array - the array to map
 * @param callback - the asynchronous map function
 * @returns the mapped array
 * @public
 */
export declare function mapAsync<T1, T2>(// tslint:disable-line:export-name
array: T1[], callback: (value: T1, index: number, array: T1[]) => Promise<T2>): Promise<T2[]>;

/**
 * Returns the maximum value of an array.
 * @param array - the array to get the maximum of
 * @returns the maximum value
 * @public
 */
export declare function max(array: number[]): number;

/**
 * Returns the mean of an array of numerical values.
 * @param array - the array to calculate the mean of
 * @returns the mean of the array
 * @public
 */
export declare function mean(array: number[]): number;

/**
 * Returns the median of an array of numerical values.
 * @param array - the array to calculate the median of
 * @returns the median of the array
 * @public
 */
export declare function median(array: number[]): number;

/**
 * Returns the minimum value of an array.
 * @param array - the array to get the minimum of
 * @returns the minimum value
 * @public
 */
export declare function min(array: number[]): number;

/**
 * Returns the mode of an array of numerical values.
 * @param array - the array to calculate the mode of
 * @returns the mode of the array
 * @public
 */
export declare function mode(array: number[]): number;

/**
 * Returns the range of an array of numerical values.
 * @param array - the array to calculate the range of
 * @returns the range of the array
 * @public
 */
export declare function range(array: number[]): number;

/**
 * Returns the sum of the values of a numerical array.
 * @param array - the array to get the sum of
 * @returns the sum of the values
 * @public
 */
export declare function sum(array: number[]): number;

export { }
