(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = global || self, factory(global.array = {}));
}(this, function (exports) { 'use strict';

    /**
     * Returns the frequency of each value in an array.
     * @param array - the array to get the frequencies of
     * @returns the frequency of each value in an array
     * @public
     */
    function frequencies(array) {
        var _frequencies = new Map();
        array.forEach(function (value) {
            _frequencies.set(value, (_frequencies.get(value) || 0) + 1);
        });
        return _frequencies;
    }

    /**
     * Returns the sum of the values of a numerical array.
     * @param array - the array to get the sum of
     * @returns the sum of the values
     * @public
     */
    function sum(array) {
        return array.reduce(function (previousValue, currentValue) { return previousValue + currentValue; });
    }

    /**
     * Returns the mean of an array of numerical values.
     * @param array - the array to calculate the mean of
     * @returns the mean of the array
     * @public
     */
    function mean(array) {
        return sum(array) / array.length;
    }
    /**
     * Returns the median of an array of numerical values.
     * @param array - the array to calculate the median of
     * @returns the median of the array
     * @public
     */
    function median(array) {
        if (array.length > 1) {
            array = array.sort();
            return (array.length + 1) % 2 === 0 ?
                array[((array.length + 1) / 2) - 1] :
                (array[Math.floor((array.length + 1) / 2) - 1] +
                    array[Math.ceil((array.length + 1) / 2) - 1]) / 2;
        }
        else {
            throw new Error('The array does not contain enough values to calculate the median.');
        }
    }
    /**
     * Returns the mode of an array of numerical values.
     * @param array - the array to calculate the mode of
     * @returns the mode of the array
     * @public
     */
    function mode(array) {
        var _frequencies = frequencies(array);
        var _max = -1;
        var _mode = -1;
        _frequencies.forEach(function (value, key) {
            if (_max < value) {
                _max = value;
                _mode = key;
            }
        });
        return _mode;
    }
    /**
     * Returns the range of an array of numerical values.
     * @param array - the array to calculate the range of
     * @returns the range of the array
     * @public
     */
    function range(array) {
        if (array.length > 1) {
            array = array.sort();
            return array[array.length - 1] - array[0];
        }
        else {
            throw new Error('The array does not contain enough values to calculate the range.');
        }
    }

    /**
     * Determines if an array contains a specified value.
     * @param array - the array to search
     * @param value - the value to search for
     * @returns true if the array contains the value; otherwise, false
     * @public
     */
    function contains(array, value) {
        return array.indexOf(value) >= 0;
    }

    function count(array, callback) {
        if (!callback) {
            return array.length;
        }
        else {
            return array.map(callback).filter(function (value) { return value; }).length;
        }
    }

    var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    };
    var __generator = (undefined && undefined.__generator) || function (thisArg, body) {
        var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f) throw new TypeError("Generator is already executing.");
            while (_) try {
                if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
                if (y = 0, t) op = [op[0] & 2, t.value];
                switch (op[0]) {
                    case 0: case 1: t = op; break;
                    case 4: _.label++; return { value: op[1], done: false };
                    case 5: _.label++; y = op[1]; op = [0]; continue;
                    case 7: op = _.ops.pop(); _.trys.pop(); continue;
                    default:
                        if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                        if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                        if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                        if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                        if (t[2]) _.ops.pop();
                        _.trys.pop(); continue;
                }
                op = body.call(thisArg, _);
            } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
            if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
        }
    };
    /**
     * Maps the values in an array asynchronously.
     * @param array - the array to map
     * @param callback - the asynchronous map function
     * @returns the mapped array
     * @public
     */
    function mapAsync(// tslint:disable-line:export-name
    array, callback) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, Promise.all(array.map(callback))];
            });
        });
    }

    var __awaiter$1 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    };
    var __generator$1 = (undefined && undefined.__generator) || function (thisArg, body) {
        var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f) throw new TypeError("Generator is already executing.");
            while (_) try {
                if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
                if (y = 0, t) op = [op[0] & 2, t.value];
                switch (op[0]) {
                    case 0: case 1: t = op; break;
                    case 4: _.label++; return { value: op[1], done: false };
                    case 5: _.label++; y = op[1]; op = [0]; continue;
                    case 7: op = _.ops.pop(); _.trys.pop(); continue;
                    default:
                        if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                        if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                        if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                        if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                        if (t[2]) _.ops.pop();
                        _.trys.pop(); continue;
                }
                op = body.call(thisArg, _);
            } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
            if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
        }
    };
    /**
     * Filters the values in an array asynchronously.
     * @param array - the array to filter
     * @param callback - the asynchronous filter function
     * @returns the filtered array
     * @public
     */
    function filterAsync(// tslint:disable-line:export-name
    array, callback) {
        return __awaiter$1(this, void 0, void 0, function () {
            var booleans;
            return __generator$1(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, mapAsync(array, callback)];
                    case 1:
                        booleans = _a.sent();
                        return [2 /*return*/, array.filter(function (value, index) { return booleans[index]; })];
                }
            });
        });
    }

    /**
     * Returns the maximum value of an array.
     * @param array - the array to get the maximum of
     * @returns the maximum value
     * @public
     */
    function max(array) {
        return array.sort(function (a, b) { return b - a; })[0];
    }

    /**
     * Returns the minimum value of an array.
     * @param array - the array to get the minimum of
     * @returns the minimum value
     * @public
     */
    function min(array) {
        return array.sort(function (a, b) { return a - b; })[0];
    }

    exports.contains = contains;
    exports.count = count;
    exports.filterAsync = filterAsync;
    exports.frequencies = frequencies;
    exports.mapAsync = mapAsync;
    exports.max = max;
    exports.mean = mean;
    exports.median = median;
    exports.min = min;
    exports.mode = mode;
    exports.range = range;
    exports.sum = sum;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.umd.js.map
